// ------------------------------------------------------------------------------
// Slopes - http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Ball.java
// Draws the balls, updates their positions, and does all the collision detection
// and collision reactions.
// ------------------------------------------------------------------------------

class Ball
{
  float x;
  float y;
  float vx;        // Velocity on the x-axis
  float vy;        // Velocity on the y-axis
  float bk;        // Bounce constant
  float fk;        // Friction constant
  
  Ball()
  {
    // Initialize random position
    x = random(0,width);
    y = random(-height*5/6,height/6);
    
    // ... with random velocity
    vy = 0;
    vx = random(0,2)-random(0,2);
    
    // Set the constants
    bk = 0.4;
    fk = 0.7;
  }
  
  void fall()
  {
    // Crude approximation of forces et al
    vy += 0.05;
  }
  
  void bounce()
  {
    // Search the obstacle's and check to see
    // if the ball is colliding with any of them    
    for ( int i=0; i<num_o; i++ )
    {
      // Speed enhancer
      if ( x >= Obstacles[i].x1 && x <= Obstacles[i].x2 ) 
      {
        // y = m*x + b
        float m = tan(Obstacles[i].t);
        float b = Obstacles[i].y2 - Obstacles[i].x2*m;
        float k = Obstacles[i].t/abs(Obstacles[i].t);
        
        // This if makes the obstacles have 'thickness'
        if ( abs(y+vy - m*(x+vx) - b) < 5 )
        {
          // Now calculate the angle it should
          // fly off to. First we find the normal
          // to the obstacle
          float md = atan(-1/m);
          
          // Then you find the reflection angle
          float t0 = atan2(-vy,-vx);
          float t1 = md + (md-t0);
          float v = sqrt(pow(vx,2)+pow(vy,2));
          
          // Change velocities
          vy = bk*v*sin(t1);
          vx = fk*v*cos(t1);
        }
      }
    }
    
    // Make sure it doesn't go off the screen
    if ( y + vy > height - 2 && vy > 0 ) 
    {
      vy *= -bk;
      vx *= fk;
    }
    if ( ( x + vx > width && vx > 0 ) || ( x + vx < 0 && vx < 0 ) ) 
    {
      vx *= -bk;
    }
  }
  
  void move()
  {
    // Update the ball's position
    x += vx;
    y += vy;
  }
  
  void display()
  {
    stroke(0,0,0);
    strokeWeight(3);
    point(x,y);
    strokeWeight(1);
  }
}
