import processing.core.*; import java.applet.*; import java.awt.*; import java.awt.image.*; import java.awt.event.*; import java.io.*; import java.net.*; import java.text.*; import java.util.*; import java.util.zip.*; public class Slopes extends PApplet {// ------------------------------------------------------------------------------
// Slopes - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Slopes.java
// Procedures related to starting up the program and adjusting the settings
// are in this file.
// ------------------------------------------------------------------------------

int num_b = 100;
int num_o = 4;
Ball[] Balls;
Obstacle[] Obstacles;

public void setup()   
{
  size(800,420);                   
  framerate(120);                    
  colorMode(RGB, 255);              
  smooth();      
  reset();
}

public void draw()                         
{
  // Clear the screen
  background(255, 255, 255);
  
  // Update and draw the balls         
  for ( int i=0; i<num_b; i++ )     
  { 
    Balls[i].fall();                  
    Balls[i].bounce();              
    Balls[i].move();                 
    Balls[i].display();          
  }
  
  // Draw the obstacles
  for ( int i=0; i<num_o; i++ )     
  {
    Obstacles[i].display();         
  }
}

public void mousePressed()
{
  reset();                        
}

public void reset()
{ 
  // Reset the balls' positions 
  Balls = new Ball[num_b];
  for ( int i=0; i<num_b; i++ )      
  {
    Balls[i] = new Ball();
  }
  
  // Create the obstacless
  Obstacles = new Obstacle[num_o];
  Obstacles[0] = new Obstacle(200,100,400,200);
  Obstacles[1] = new Obstacle(400,250,600,150);
  Obstacles[2] = new Obstacle(200,200,400,300);
  Obstacles[3] = new Obstacle(350,375,600,250);
}
    

// ------------------------------------------------------------------------------
// Slopes - http://www.arkitus.com
// Copyright (C) 2006  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Ball.java
// Draws the balls, updates their positions, and does all the collision detection
// and collision reactions.
// ------------------------------------------------------------------------------

class Ball
{
  float x;
  float y;
  float vx;        // Velocity on the x-axis
  float vy;        // Velocity on the y-axis
  float bk;        // Bounce constant
  float fk;        // Friction constant
  
  Ball()
  {
    // Initialize random position
    x = random(0,width);
    y = random(-height*5/6,height/6);
    
    // ... with random velocity
    vy = 0;
    vx = random(0,2)-random(0,2);
    
    // Set the constants
    bk = 0.4f;
    fk = 0.7f;
  }
  
  public void fall()
  {
    // Crude approximation of forces et al
    vy += 0.05f;
  }
  
  public void bounce()
  {
    // Search the obstacle's and check to see
    // if the ball is colliding with any of them    
    for ( int i=0; i<num_o; i++ )
    {
      // Speed enhancer
      if ( x >= Obstacles[i].x1 && x <= Obstacles[i].x2 ) 
      {
        // y = m*x + b
        float m = tan(Obstacles[i].t);
        float b = Obstacles[i].y2 - Obstacles[i].x2*m;
        float k = Obstacles[i].t/abs(Obstacles[i].t);
        
        // This if makes the obstacles have 'thickness'
        if ( abs(y+vy - m*(x+vx) - b) < 5 )
        {
          // Now calculate the angle it should
          // fly off to. First we find the normal
          // to the obstacle
          float md = atan(-1/m);
          
          // Then you find the reflection angle
          float t0 = atan2(-vy,-vx);
          float t1 = md + (md-t0);
          float v = sqrt(pow(vx,2)+pow(vy,2));
          
          // Change velocities
          vy = bk*v*sin(t1);
          vx = fk*v*cos(t1);
        }
      }
    }
    
    // Make sure it doesn't go off the screen
    if ( y + vy > height - 2 && vy > 0 ) 
    {
      vy *= -bk;
      vx *= fk;
    }
    if ( ( x + vx > width && vx > 0 ) || ( x + vx < 0 && vx < 0 ) ) 
    {
      vx *= -bk;
    }
  }
  
  public void move()
  {
    // Update the ball's position
    x += vx;
    y += vy;
  }
  
  public void display()
  {
    stroke(0,0,0);
    strokeWeight(3);
    point(x,y);
    strokeWeight(1);
  }
}

// ------------------------------------------------------------------------------
// Slopes - http://www.arkitus.com
// Copyright (C) 2005  Ali Eslami
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// ------------------------------------------------------------------------------
// Obstacle.java
// The slopes. Actual collision detection happens in Ball.java.
// ------------------------------------------------------------------------------

class Obstacle
{
  float x1;
  float y1;
  float x2;
  float y2;
  float t;
    
  Obstacle(float X1, float Y1, float X2, float Y2)
  {
      x1 = X1;
      y1 = Y1;
      x2 = X2;
      y2 = Y2;

      t = atan((y2-y1)/(x2-x1));
  }
  
  public void display()
  {
    stroke(220,220,220);
    strokeWeight(5);
    line(x1,y1,x2,y2);
    strokeWeight(1);
  }
}
static public void main(String args[]) {   PApplet.main(new String[] { "Slopes" });}}